function [H] = Hmatrix(node,elem,mg,mp,M,boundary)
%[H] = Hmatrix(node,elem,m,M)
%     H: The viscosity matrix
%  node: The nodal positions
%  elem: The element connectivities
%   m,M: Mobilities

% Mobility of grain boundary (m,mg,mb,Mb) and Pore surface(ms,mp,Ms,M)
m=mg;
ms=mp;
Ms=M;
mb=m;

%The number of elements
nel=size(elem,1);

%Number of nodes
nnode=size(node,1);

%Find the number of degrees of freedom. This is also the size of the H
%maxtrix.
Hsize=nel+3*nnode;

%Initialize the H matrix
H=zeros(Hsize,Hsize);

%Used to add 1/Mx and 1/My to the elemental matrix. Used to ensure the H
%matrix is not singular
add=[[1 0 0 0 0 0 0];
     [0 1 0 0 0 0 0];
     [0 0 0 0 0 0 0];
     [0 0 0 1 0 0 0];
     [0 0 0 0 1 0 0];
     [0 0 0 0 0 0 0];
     [0 0 0 0 0 0 0]];

 
for i=1:nel

    %The two nodes which comprise the element
    node1=elem(i,1);
    node2=elem(i,2);
    
    %Positions of the ends of the element
    x1=node(node1,1);
    y1=node(node1,2);
    x2=node(node2,1);
    y2=node(node2,2);
    
    %The length of the element
    l=sqrt((y2-y1)^2+(x2-x1)^2);
    
    %Find the Sine and Cosine of the element angle
    S=(y2-y1)/l;
    C=(x2-x1)/l;           
    
    %% Surface boundary element viscosity 7*7 matrix
H11=(pi*S^2*l*(2*x1 + 2*x2 - C*l))/(6*ms);
H12=-(pi*C*S*l*(2*x1 + 2*x2 - C*l))/(6*ms);
H13=(pi*S*(5*x1 + 5*x2 - 3*C*l))/(6*ms);
H14=(pi*S^2*l*(x1 + x2))/(6*ms);
H15= -(pi*C*S*l*(x1 + x2))/(6*ms);
H16=-(pi*S*(x1 + x2 + C*l))/(6*ms);
H17=-(2*pi*S*(x1 + x2 - C*l))/(3*ms);

H22=(pi*C^2*l*(2*x1 + 2*x2 - C*l))/(6*ms);
H23=-(pi*C*(5*x1 + 5*x2 - 3*C*l))/(6*ms);
H24=-(pi*C*S*l*(x1 + x2))/(6*ms);
H25=(pi*C^2*l*(x1 + x2))/(6*ms);
H26=(pi*C*(x1 + x2 + C*l))/(6*ms);
H27=(2*pi*C*(x1 + x2 - C*l))/(3*ms);

H33=(pi*l*(4*x1 + 4*x2 - 3*C*l))/(30*Ms) - (pi*((4*C)/3 - ((7*x1)/3 + (7*x2)/3)/l))/ms;
H34=(pi*S*(x1 + x2 - C*l))/(6*ms);
H35=-(pi*C*(x1 + x2 - C*l))/(6*ms);
H36=(pi*(x1 + x2))/(3*l*ms) - (pi*l*(x1 + x2))/(30*Ms);
H37=(pi*((4*C)/3 - ((8*x1)/3 + (8*x2)/3)/l))/ms + (pi*l*(x1 + x2 - C*l))/(15*Ms);

H44=(pi*S^2*l*(2*x1 + 2*x2 + C*l))/(6*ms);
H45=-(pi*C*S*l*(2*x1 + 2*x2 + C*l))/(6*ms);
H46=-(pi*S*(5*x1 + 5*x2 + 3*C*l))/(6*ms);
H47=(2*pi*S*(x1 + x2 + C*l))/(3*ms);

H55=(pi*C^2*l*(2*x1 + 2*x2 + C*l))/(6*ms);
H56=(pi*C*(5*x1 + 5*x2 + 3*C*l))/(6*ms);
H57=-(2*pi*C*(x1 + x2 + C*l))/(3*ms);

H66=(pi*((4*C)/3 + ((7*x1)/3 + (7*x2)/3)/l))/ms + (pi*l*(4*x1 + 4*x2 + 3*C*l))/(30*Ms);
H67=(pi*l*(x1 + x2 + C*l))/(15*Ms) - (pi*((4*C)/3 + ((8*x1)/3 + (8*x2)/3)/l))/ms;

H77=(8*pi*l*(x1 + x2))/(15*Ms) + (16*pi*(x1 + x2))/(3*l*ms);


Hs = [H11 H12 H13 H14 H15 H16 H17;
      H12 H22 H23 H24 H25 H26 H27;
      H13 H23 H33 H34 H35 H36 H37;
      H14 H24 H34 H44 H45 H46 H47;
      H15 H25 H35 H45 H55 H56 H57;
      H16 H26 H36 H46 H56 H66 H67;
      H17 H27 H37 H47 H57 H67 H77];

%% Grain boundary element viscosity 4*4 matrix 
Mb=1e-6; % Use this small value to to change the 4*4 matrix into 7*7 matrix by letting the diffusion mobility on grain boundary as a small number

H11b=(pi*S^2*l*(2*x1 + 2*x2 - C*l))/(6*mb);
H12b=-(pi*C*S*l*(2*x1 + 2*x2 - C*l))/(6*mb);
H13b=(pi*S*(5*x1 + 5*x2 - 3*C*l))/(6*mb);
H14b=(pi*S^2*l*(x1 + x2))/(6*mb);
H15b= -(pi*C*S*l*(x1 + x2))/(6*mb);
H16b=-(pi*S*(x1 + x2 + C*l))/(6*mb);
H17b=-(2*pi*S*(x1 + x2 - C*l))/(3*mb);

H22b=(pi*C^2*l*(2*x1 + 2*x2 - C*l))/(6*mb);
H23b=-(pi*C*(5*x1 + 5*x2 - 3*C*l))/(6*mb);
H24b=-(pi*C*S*l*(x1 + x2))/(6*mb);
H25b=(pi*C^2*l*(x1 + x2))/(6*mb);
H26b=(pi*C*(x1 + x2 + C*l))/(6*mb);
H27b=(2*pi*C*(x1 + x2 - C*l))/(3*mb);

H33b=(pi*l*(4*x1 + 4*x2 - 3*C*l))/(30*Mb) - (pi*((4*C)/3 - ((7*x1)/3 + (7*x2)/3)/l))/mb;
H34b=(pi*S*(x1 + x2 - C*l))/(6*mb);
H35b=-(pi*C*(x1 + x2 - C*l))/(6*mb);
H36b=(pi*(x1 + x2))/(3*l*mb) - (pi*l*(x1 + x2))/(30*Mb);
H37b=(pi*((4*C)/3 - ((8*x1)/3 + (8*x2)/3)/l))/mb + (pi*l*(x1 + x2 - C*l))/(15*Mb);

H44b=(pi*S^2*l*(2*x1 + 2*x2 + C*l))/(6*mb);
H45b=-(pi*C*S*l*(2*x1 + 2*x2 + C*l))/(6*mb);
H46b=-(pi*S*(5*x1 + 5*x2 + 3*C*l))/(6*mb);
H47b=(2*pi*S*(x1 + x2 + C*l))/(3*mb);

H55b=(pi*C^2*l*(2*x1 + 2*x2 + C*l))/(6*mb);
H56b=(pi*C*(5*x1 + 5*x2 + 3*C*l))/(6*mb);
H57b=-(2*pi*C*(x1 + x2 + C*l))/(3*mb);

H66b=(pi*((4*C)/3 + ((7*x1)/3 + (7*x2)/3)/l))/mb + (pi*l*(4*x1 + 4*x2 + 3*C*l))/(30*Mb);
H67b=(pi*l*(x1 + x2 + C*l))/(15*Mb) - (pi*((4*C)/3 + ((8*x1)/3 + (8*x2)/3)/l))/mb;

H77b=(8*pi*l*(x1 + x2))/(15*Mb) + (16*pi*(x1 + x2))/(3*l*mb);

Hb1 = [H11b H12b H13b H14b H15b H16b H17b;
      H12b H22b H23b H24b H25b H26b H27b;
      H13b H23b H33b H34b H35b H36b H37b;
      H14b H24b H34b H44b H45b H46b H47b;
      H15b H25b H35b H45b H55b H56b H57b;
      H16b H26b H36b H46b H56b H66b H67b;
      H17b H27b H37b H47b H57b H67b H77b];
    %% Adding large mobilities (1/mx), (1/my) at nodes to discourage singularity
    
    He=zeros(7,7);
    if elem(i,3)==0;  %%the element is on grain boundary (L=1, (L/h*mx) is in the range of 1e-6 to 1e-3)
        He=Hb1*M+1e-4*add;        
    elseif elem(i,3)==1; %% the element is on pore surface (L=1e-6, (L/h*mx) is in the 1e-6 to 1e-3)
        He=Hs*M+(1e0)*add;    
    else
        disp('error when selecting H matrix');
    end               
    
    %Now assemble the elemental matrix into the global one
        
        n1=elem(i,1);
        n2=elem(i,2);
        H(3*n1-2:3*n1,3*n1-2:3*n1)=H(3*n1-2:3*n1,3*n1-2:3*n1)+He(1:3,1:3);
        H(3*n2-2:3*n2,3*n2-2:3*n2)=H(3*n2-2:3*n2,3*n2-2:3*n2)+He(4:6,4:6);
        H(3*n1-2:3*n1,3*n2-2:3*n2)=H(3*n1-2:3*n1,3*n2-2:3*n2)+He(1:3,4:6);
        H(3*n2-2:3*n2,3*n1-2:3*n1)=H(3*n2-2:3*n2,3*n1-2:3*n1)+He(4:6,1:3);
        H(3*n1-2:3*n1,3*nnode+i)=H(3*n1-2:3*n1,3*nnode+i)+He(1:3,7);
        H(3*n2-2:3*n2,3*nnode+i)=H(3*n2-2:3*n2,3*nnode+i)+He(4:6,7);
        H(3*nnode+i,3*n1-2:3*n1)=H(3*nnode+i,3*n1-2:3*n1)+He(7,1:3);
        H(3*nnode+i,3*n2-2:3*n2)=H(3*nnode+i,3*n2-2:3*n2)+He(7,4:6);
        H(3*nnode+i,3*nnode+i)=H(3*nnode+i,3*nnode+i)+He(7,7);   
        
    % Ensure fixed boundary conditions by adding large number    
        if i==boundary(1,1)
            H(3*n2-2,3*n2-2)=H(3*n2-2,3*n2-2)+1e50;
            H(3*n2,3*n2)=H(3*n2,3*n2)+1e50;
        elseif i==boundary(1,2)
            H(3*n2-2,3*n2-2)=H(3*n2-2,3*n2-2)+1e50;
            H(3*n2,3*n2)=H(3*n2,3*n2)+1e50;
        elseif i==boundary(1,3)
            H(3*n2-2,3*n2-2)=H(3*n2-2,3*n2-2)+1e50;           
        end
        
     
end

