function [node,elem,flag,boundary] = check(node,elem,minel,maxel,boundary)
%[node,elem] = check(node,elem,minel,maxel)
%  node: The nodal positions
%  elem: The element connectivities
% minel: The minimum element size
% maxel: The maximum element size

%Set the initial value for the flag. 0 is good
flag=0;

%The sizes
nel=size(elem,1);
nnode=size(node,1);

%Determines the length and angle of each element
[l,angle]=len(node,elem);

%% If any elements are too small, then fix them
while and(min(l)<minel,flag==0)
% Find element with min length
    [y,i]=min(l);   

% Exchange first node of element i with the final node
   temp=elem(i,1);
   temp1=node(temp,:);
   node(temp,:)=node(nnode,:);
   node(nnode,:)=temp1;
   ntemp1=[]; ntemp2=[];
   nfinal1=[]; nfinal2=[]; 
   for k=1:nel       
       if (elem(k,1)==temp)
           ntemp1=[ntemp1,k];
       end
       if (elem(k,2)==temp)
           ntemp2=[ntemp2,k];           
       end
   end
   for k=1:nel
       if (elem(k,1)==nnode)
           nfinal1=[nfinal1,k];
       elseif (elem(k,2)==nnode)
           nfinal2=[nfinal2,k];
       end
   end
   
   for k=1:length(ntemp1)
       n1=ntemp1(k);
       elem(n1,1)=nnode;
   end
   for k=1:length(ntemp2)
       n2=ntemp2(k);
       elem(n2,2)=nnode;
   end
   for k=1:length(nfinal1)
       n1=nfinal1(k);
       elem(n1,1)=temp;
   end
   for k=1:length(nfinal2)
       n2=nfinal2(k);
       elem(n2,2)=temp;
   end
  
   for k=1:nel
       n1=elem(k,1); n2=elem(k,2);
       if and(n1==nnode,k~=i)           
           elem(k,1)=elem(i,2);           
       end
       if and(n2==nnode,k~=i)
           elem(k,2)=elem(i,2);
       end       
   end
      
% Exchange element i and the final element   
   temp2=elem(i,:);
   elem(i,:)=elem(nel,:);
   elem(nel,:)=temp2;
% Change boundary if necessary
           if i==boundary(1,1)
               for k=1:nel-1
                   if elem(k,2)==elem(nel,2)
                       boundary(1,1)=k;
                   end
               end
           end
           if i==boundary(1,2)
               for k=1:nel-1
                   if elem(k,2)==elem(nel,2)
                       boundary(1,2)=k;
                   end                   
               end
           end
           if i==boundary(1,3)
               for k=1:nel-1
                   if elem(k,2)==elem(nel,2)
                       boundary(1,3)=k;
                   end                   
               end
           end

% Delete the final element and final node   
    node=node(1:nnode-1,1:2);
    elem=elem(1:nel-1,1:3);
    
    nel=size(elem,1);
    nnode=size(node,1);
    
    [l,angle]=len(node,elem);
    
    if nel<3
        flag=1;
    end
   
end

%% IF any elements are too big, then split them
while and(max(l)>maxel,flag==0)
    [y,i]=max(l);
    
    
    node1=elem(i,1);
    node2=elem(i,2);
    
    x1=node(node1,1);
    y1=node(node1,2);
    x2=node(node2,1);
    y2=node(node2,2);
% Break the elements in to two elements    
    temp=zeros(nnode+1,2);
    temp1=zeros(nel+1,3);         
      temp(1:nnode,1:2)=node;      
      temp(nnode+1,1)=(x1+x2)/2;
      temp(nnode+1,2)=(y1+y2)/2;
      temp1(1:nel,1:3)=elem;
      temp1(i,2)=nnode+1;
      temp1(nel+1,1)=nnode+1;
      temp1(nel+1,2)=elem(i,2);
      temp1(nel+1,3)=elem(i,3);
      
      node=temp;
      elem=temp1;

    nel=size(elem,1);
    nnode=size(node,1);
% Change the boundary if necessary    
    if i==boundary(1,3)
        boundary(1,3)=nel;
    elseif i==boundary(1,2);
        boundary(1,2)=nel;
    elseif i==boundary(1,1)
        boundary(1,1)=nel;
    end
    
    [l,angle]=len(node,elem);
 
end

