function [f] = fvector(node,elem,g0,ki,boundary)
%[f] = fvector(node,elem,g0,ki)
%    f: The force vector
% node: Nodal positions
% elem: Element connectivities
%   g0: The surface tension
%   ki: Used for anisotropy


%% fb the force on the two ends
fb=g0/2;    % y direction

% Surface tension on grain boundary and pore surface
gammaB=g0;
gammaS=g0;

%Number of elements
nel=size(elem,1);

%Number of nodes
nnode=size(node,1);

%Initialize the f vector
f=zeros(nel+3*nnode,1);


for i=1:nel;
    
    %The nodes on either side of the element
    node1=elem(i,1);
    node2=elem(i,2);
    
    %The nodal positions
    x1=node(node1,1);
    y1=node(node1,2);
    x2=node(node2,1);
    y2=node(node2,2);
    
    %Length of the element
    l=sqrt((y2-y1)^2+(x2-x1)^2);
    
    %The Sine and Cosine of the element angle
    S=(y2-y1)/l;
    C=(x2-x1)/l;
    
%% Find the forces at the node.
    % added to select the right gamma
 if elem(i,3)==0;  %%the element is on grain boundary
       gamma=gammaB;    
    elseif elem(i,3)==1; %% the element is on surface
       gamma=gammaS;
    else
        disp('error when selecting gamma');
 end
       
    %Assemble the global f vector       
        n1=elem(i,1);
        n2=elem(i,2);
        f(3*n1-2)=f(3*n1-2)+gamma*pi*(-l+(x1+x2)*C)/gamma;
        f(3*n1-1)=f(3*n1-1)+gamma*pi*((x1+x2)*S)/gamma;
        f(3*n2-2)=f(3*n2-2)+gamma*pi*(-l-(x1+x2)*C)/gamma;
        f(3*n2-1)=f(3*n2-1)+gamma*pi*(-(x1+x2)*S)/gamma;
        
    %Add force at boundary: 
          if i==boundary(1,3)
          f(3*n2-1)=f(3*n2-1)+5*pi*fb/gamma;           
          end    
    
end
